//
//  LBCViewController.m
//  LBCKipptKitSandbox
//
//  Created by Ben Longoria on 7/4/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "LBCViewController.h"
#import "LBCKipptKit.h"
#import "LBCKipptAccount.h"
#import "LBCKipptList.h"
#import "LBCKipptClip.h"
#import "LBCKipptResult.h"

@implementation LBCViewController

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (IBAction)performKipptLogin:(id)sender {
    NSString *username = [usernameInput text];
    NSString *password = [passwordInput text];
    if([username length] == 0 || [password length] == 0)
        return;
    
    void (^completionBlock)(LBCKipptAccount *account, NSError *error) =
        ^(LBCKipptAccount *account, NSError *error) {
            [self handleLoginWithAccount:account error:error];
        };
    [LBCKipptKit accountWithUsername:username
                            password:password
                          completion:completionBlock];
    [[self view] endEditing:YES];
}

- (void)handleLoginWithAccount:(LBCKipptAccount *)account error:(NSError *)error {
    if(error) {
        NSLog(@"Received error for login: %@", error);
        return;
    }
    NSLog(@"LBCKipptAccount: %@", [account values]);
    _account = account;
    [getListsButton setHidden:NO];
    [getClipsButton setHidden:NO];
    [searchButton setHidden:NO];
    [searchInput setHidden:NO];
    [listTitleInput setHidden:NO];
    [postListButton setHidden:NO];
    [clipURLInput setHidden:NO];
    [postClipButton setHidden:NO];
}

- (IBAction)getLists:(id)sender {
    void (^completionBlock)(LBCKipptResult *result, NSError *error) =
        ^(LBCKipptResult *result, NSError *error) {
            [self handleResponseWithLists:result.objects error:error];
        };
    [LBCKipptKit listsWithAccount:_account
                          options:nil
                       completion:completionBlock];
}

- (void)handleResponseWithLists:(NSArray *)lists error:(NSError *)error {
    if(error || !lists || ![lists lastObject]) {
        NSLog(@"Received error or empty list for lists request: %@", error);
        return;
    }
    for(LBCKipptList *list in lists)
        NSLog(@"List: %@", [list values]);
    _listIDToFetch = [[lists objectAtIndex:0] remoteObjectID];
    [getListButton setHidden:NO];
}

- (IBAction)getList:(id)sender {
    void (^completionBlock)(LBCKipptList *list, NSError *error) =
        ^(LBCKipptList *list, NSError *error) {
            [self handleResponseWithList:list error:error];
        };
    [LBCKipptKit listWithID:_listIDToFetch account:_account completion:completionBlock];
}

- (void)handleResponseWithList:(LBCKipptList *)list error:(NSError *)error {
    if(error || !list) {
        NSLog(@"Received error or empty list for list request: %@", error);
        return;
    }
    NSLog(@"LBCKipptList: %@", [list values]);
}

- (IBAction)getClips:(id)sender {
    void (^completionBlock)(LBCKipptResult *result, NSError *error) =
        ^(LBCKipptResult *result, NSError *error) {
            [self handleResponseWithClips:result.objects error:error];
        };
    NSMutableDictionary *options = [NSMutableDictionary dictionary];
    [options setObject:@"3" forKey:LBCK_OPTION_LIMIT];
    [options setObject:LBCK_OPTION_LIST forKey:LBCK_OPTION_INCLUDE_DATA];
    [LBCKipptKit clipsWithAccount:_account
                          options:options
                       completion:completionBlock];
}

- (void)handleResponseWithClips:(NSArray *)clips error:(NSError *)error {
    if(error || !clips || ![clips lastObject]) {
        NSLog(@"Received error or empty list for clips/search request: %@", error);
        return;
    }
    for(LBCKipptClip *clip in clips)
        NSLog(@"Clip: %@", [clip values]);
    _clipIDToFetch = [[clips objectAtIndex:0] remoteObjectID];
    [getClipButton setHidden:NO];
}

- (IBAction)getClip:(id)sender {
    void (^completionBlock)(LBCKipptClip *clip, NSError *error) =
        ^(LBCKipptClip *clip, NSError *error) {
            [self handleResponseWithClip:clip error:error];
        };
    [LBCKipptKit clipWithID:_clipIDToFetch account:_account completion:completionBlock];
}

- (void)handleResponseWithClip:(LBCKipptClip *)clip error:(NSError *)error {
    if(error || !clip) {
        NSLog(@"Received error or empty clip for clip request: %@", error);
        return;
    }
    NSLog(@"LBCKipptClip: %@", [clip values]);
}

- (IBAction)searchForClips:(id)sender {
    void (^completionBlock)(LBCKipptResult *result, NSError *error) =
        ^(LBCKipptResult *result, NSError *error) {
            [self handleResponseWithClips:result.objects error:error];
        };
    [LBCKipptKit searchWithQuery:[searchInput text]
                         account:_account
                         options:nil
                      completion:completionBlock];
    [[self view] endEditing:YES];
}

- (IBAction)postList:(id)sender {
    void (^completionBlock)(LBCKipptList *list, NSError *error) =
        ^(LBCKipptList *list, NSError *error) {
            [self handlePostResponseWithList:list error:error];
        };
    LBCKipptList *listToPost = [[LBCKipptList alloc] init];
    listToPost.title = [listTitleInput text];
    [LBCKipptKit saveObject:listToPost withAccount:_account completion:completionBlock];
    [[self view] endEditing:YES];
}

- (void)handlePostResponseWithList:(LBCKipptList *)list error:(NSError *)error {
    if(error || !list) {
        NSLog(@"Received error or empty list for list request: %@", error);
        return;
    }
    _listToManhandle = list;
    [deleteListButton setHidden:NO];
    [updateListButton setHidden:NO];
    NSLog(@"LBCKipptList: %@", [list values]);
}

- (IBAction)postClip:(id)sender {
    void (^completionBlock)(LBCKipptClip *clip, NSError *error) =
        ^(LBCKipptClip *clip, NSError *error) {
            [self handlePostResponseWithClip:clip error:error];
        };
    LBCKipptClip *clipToPost = [[LBCKipptClip alloc] init];
    clipToPost.url = [clipURLInput text];
    [LBCKipptKit saveObject:clipToPost withAccount:_account completion:completionBlock];
    [[self view] endEditing:YES];
    
}

- (void)handlePostResponseWithClip:(LBCKipptClip *)clip error:(NSError *)error {
    if(error || !clip) {
        NSLog(@"Received error or empty clip for clip request: %@", error);
        return;
    }
    _clipToManhandle = clip;
    [deleteClipButton setHidden:NO];
    [updateClipButton setHidden:NO];
    [favorClipButton setHidden:NO];
    NSLog(@"LBCKipptClip: %@", [clip values]);
}

- (IBAction)deleteClip:(id)sender {
    [self deleteKipptObject:_clipToManhandle];
    [deleteClipButton setHidden:YES];
    [updateClipButton setHidden:YES];
}

- (IBAction)deleteList:(id)sender {
    [self deleteKipptObject:_listToManhandle];
    [deleteListButton setHidden:YES];
    [updateListButton setHidden:YES];
}

- (void)deleteKipptObject:(id <LBCKipptObject>)object {
    void (^completionBlock)(NSNumber *responseCode, NSError *error) =
    ^(NSNumber *responseCode, NSError *error) {
        [self handleDeleteResponseWithCode:responseCode error:error];
    };
    [LBCKipptKit deleteObject:object withAccount:_account completion:completionBlock];
}

- (void)handleDeleteResponseWithCode:(NSNumber *)code error:(NSError *)error {
    if(error || !code) {
        NSLog(@"Received error or empty response for post: %@", error);
        return;
    }
    NSLog(@" Delete response code: %@", code);
}

- (IBAction)updateClip:(id)sender {
    LBCKipptClip *clip = (LBCKipptClip *)_clipToManhandle;
    clip.notes = @"Updates notes!";
    [self updateKipptObject:clip];
}

- (IBAction)updateList:(id)sender {
    LBCKipptList *list = (LBCKipptList *)_listToManhandle;
    list.title = [list.title stringByAppendingString:@" [UPDATED]"];
    [self updateKipptObject:list];
}

- (IBAction)favorClip:(id)sender {
    void (^completionBlock)(NSNumber *success, NSError *error) =
        ^(NSNumber *favoriteStatus, NSError *error) {
            [self handleFavorResponseWithStatus:favoriteStatus error:error];
        };
    if(_clipToManhandle.isFavorite) {
        [LBCKipptKit unfavorClip:_clipToManhandle withAccount:_account completion:completionBlock];
    } else {
        [LBCKipptKit favorClip:_clipToManhandle withAccount:_account completion:completionBlock];
    }
    
}

- (void)handleFavorResponseWithStatus:(NSNumber *)favoriteStatus error:(NSError *)error {
    if(error) {
        NSLog(@"Received error for favor: %@", error);
        return;
    }
    _clipToManhandle.favorite = [favoriteStatus boolValue];
    [favorClipButton setTitle:(_clipToManhandle.isFavorite) ? @"Unfavor Clip" : @"Favor Clip" forState:UIControlStateNormal];
    NSLog(@"Favored status: %@", [favoriteStatus boolValue] ? @"true" : @"false");
}

- (void)updateKipptObject:(id <LBCKipptObject>)object {
    void (^completionBlock)(id <LBCKipptObject> object, NSError *error) =
        ^(id <LBCKipptObject> object, NSError *error) {
            [self handlePutWithObject:object error:error];
        };
    [LBCKipptKit saveObject:object withAccount:_account completion:completionBlock];
}

- (void)handlePutWithObject:(id <LBCKipptObject>)object error:(NSError *)error {
    if(error || !object) {
        NSLog(@"Received error or empty object for update: %@", error);
        return;
    }
    NSLog(@"Updated LBCKipptObject instance: %@", [object values]);
}

#pragma mark UITextFieldDelegate methods
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if(textField == usernameInput || textField == passwordInput) {
        [self performKipptLogin:nil];
    } else if(textField == searchInput) {
        [self searchForClips:nil];
    } else if(textField == listTitleInput) {
        [self postList:nil];
    } else if(textField == clipURLInput) {
        [self postClip:nil];
    }
    return YES;
}

@end
